import { HashMap, Translation } from '@ngneat/transloco';
import { TranslationMarkupRenderer } from './translation-markup-renderer.model';
import { TranslationMarkupTranspiler } from './translation-markup-transpiler.model';
/**
 * Creates a translation markup rendering function for the specified translation value.
 *
 * @param   translationValue Translation value containing markup tokens that is to be converted to a rendering function.
 * @param   transpilers      Ordered set of transpiler that are available for the conversion.
 * @param   translation      Translation dictionary containing the translation value.
 * @returns                  A rendering function that can render the translation given translation parameter to a specified target element.
 */
export declare function createTranslationMarkupRenderer(translationValue: string, transpilers: TranslationMarkupTranspiler[], translation: Translation): (target: Element, translationParameters: HashMap) => void;
/**
 * Converts the given translation value into a sequence of tokens that can be parsed by the specified transpilers. Note that the order of
 * the transpilers is used to resolve conflicts when two or more transpilers can recognize a token at a specific location within the
 * translation value. Tokens are generated based on the first transpiler that is able to create token at a certain offset.
 *
 * @param   translationValue Translation value that is to be tokenized.
 * @param   transpilers      Ordered set transpilers used to convert the translation value into tokens.
 * @returns                  A sequence of tokens which can be parsed by the given transpilers and converted into markup renderers.
 */
export declare function tokenize(translationValue: string, transpilers: TranslationMarkupTranspiler[]): unknown[];
/**
 * Transpiles the given sequence of tokens using the provded transpilers into a `TranslationMarkupRenderer`.
 *
 * @param   tokens      Token sequence that is to be transpiled.
 * @param   transpilers Transpilers which are to be used to parse and convert the token sequence.
 * @param   translation Translation dictionary containing the translation value that is represented by the given tokens.
 * @returns             A `TranslationMarkupRenderer` that was constructed by the transpilers from the token sequence.
 */
export declare function transpile(tokens: unknown[], transpilers: TranslationMarkupTranspiler[], translation: Translation): TranslationMarkupRenderer[];
