import { ResolveLinkSpecification } from '../models/resolve-link-specification.model';
import { LinkRenderer } from '../link-renderer.model';
import { TranslationMarkupRendererFactory } from '../translation-markup-renderer-factory';
import { ContextualLinkBlockTranspiler } from './contextual-link-block-transpiler';
import { ContextualLinkSubstitutionTranspiler, ContextualLinkSubstitutionTranspilerOptions } from './contextual-link-substitution-transpiler';
import * as i0 from "@angular/core";
/**
 * Injectable factory service that provides convenience functions for easily creating contextual link transpiler instances.
 */
export declare class ContextualLinkTranspilerFactory {
    /** Renderer factory used to create the renderer for the HTML anchor element that will be substituted in place of the token. */
    private readonly rendererFactory;
    /** Set of link renderers that are used to apply the link model from the translation parameter to the anchor element. */
    private readonly linkRenderers;
    /**
     * Creates a new `ContextualLinkTranspilerFactory` that uses the specified renderer factory and link renderers for the creation of
     * `ContextualLinkBlockTranspiler` and `ContextualLinkSubstitutionTranspiler` instances.
     */
    constructor(
    /** Renderer factory used to create the renderer for the HTML anchor element that will be substituted in place of the token. */
    rendererFactory: TranslationMarkupRendererFactory, 
    /** Set of link renderers that are used to apply the link model from the translation parameter to the anchor element. */
    linkRenderers: LinkRenderer<unknown> | LinkRenderer<unknown>[] | null);
    /**
     * Creates a `ContextualLinkBlockTranspiler` that resolves the link based on the specified parameter key. The start and end tokens are
     * also based on the parameter key, which are equal to `[parameterKey]` and `[/parameterKey]`.
     *
     * @param   parameterKey Key of the translation parameter which holds the `link` model.
     * @returns              The `ContextualLinkSubstitutionTranspiler` that was created.
     */
    createBlockTranspiler(parameterKey: string): ContextualLinkBlockTranspiler;
    /**
     * Creates a `ContextualLinkSubstitutionTranspiler` for the specified tokens and link specification resolver.
     *
     * @param   startToken               Token that defined the start of the link.
     * @param   endToken                 Token that defined end start of the link.
     * @param   resolveLinkSpecification Object that specifies how to resolve the link.
     * @returns                          The `ContextualLinkBlockTranspiler` that was created.
     */
    createBlockTranspiler(startToken: string, endToken: string, resolveLinkSpecification: ResolveLinkSpecification): ContextualLinkBlockTranspiler;
    /**
     * Creates a `ContextualLinkSubstitutionTranspiler` that resolves the label and link based on the specified parameter key. The
     * translation parameter value is expected to have a `label` and `link` property. The substitution token is equal to the parameter key
     * wrapped between square brackets: `[parameterKey]`.
     *
     * @param   parameterKey Key of the translation parameter which holds the `label` and `link` subproperties.
     * @returns              The `ContextualLinkSubstitutionTranspiler` that was created.
     */
    createSubstitutionTranspiler(parameterKey: string): ContextualLinkSubstitutionTranspiler;
    /**
     * Creates a `ContextualLinkSubstitutionTranspiler` for the specified substitution token and given options.
     *
     * @param   token   The token which will be substituted with a link.
     * @param   options Options that define how to resolve the label text and link model.
     * @returns         The `ContextualLinkSubstitutionTranspiler` that was created.
     */
    createSubstitutionTranspiler(token: string, options: ContextualLinkSubstitutionTranspilerOptions): ContextualLinkSubstitutionTranspiler;
    static ɵfac: i0.ɵɵFactoryDeclaration<ContextualLinkTranspilerFactory, [null, { optional: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<ContextualLinkTranspilerFactory>;
}
