import { LinkRenderer } from '../link-renderer.model';
import { TranslationMarkupRendererFactory } from '../translation-markup-renderer-factory';
import { TranslationMarkupRenderer } from '../translation-markup-renderer.model';
import { TokenizeResult, TranslationMarkupTranspiler, TranspileResult, TranslationMarkupTranspilerContext } from '../translation-markup-transpiler.model';
import * as i0 from "@angular/core";
/**
 * A transpiler that can be used to parse links in a translation value and convert those to a renderer that generates an HTML anchor
 * element. Supports the following syntax: `[link:parameterKey]...[/link]`, where `parameterKey` should be substituted with the actual key
 * of the parameter that stores the target link. Contents between the opening and closing tags will be recursively transpiled.
 */
export declare class LinkTranspiler implements TranslationMarkupTranspiler {
    /** Renderer factory which will be used for creating the HTML anchor element renderer. */
    private readonly rendererFactory;
    /** Set of link renderers that are used to apply the link model from the translation parameter to the anchor element. */
    private readonly linkRenderers;
    /**
     * Creates a new `LinkTranspiler` that uses the specified renderer factory for creating the HTML anchor element (`<a>`) and which
     * delegates the rendering of link models to the given set of link renderers.
     */
    constructor(
    /** Renderer factory which will be used for creating the HTML anchor element renderer. */
    rendererFactory: TranslationMarkupRendererFactory, 
    /** Set of link renderers that are used to apply the link model from the translation parameter to the anchor element. */
    linkRenderers: LinkRenderer<unknown> | LinkRenderer<unknown>[] | null);
    /** @inheritdoc */
    tokenize(translation: string, offset: number): TokenizeResult | undefined;
    /** @inheritdoc */
    transpile(start: number, context: TranslationMarkupTranspilerContext): TranspileResult | undefined;
    /**
     * Creates a renderer that generates an HTML anchor tag and applies the link model stored in the specified translation parameter.
     *
     * @param   parameterKey   Key that identifies the translation parameter containing the link model.
     * @param   childRenderers Transpiled content of the link.
     * @returns                A markup renderer that creates an HTML representation of the link.
     */
    protected createRenderer(parameterKey: string, childRenderers: TranslationMarkupRenderer[]): TranslationMarkupRenderer;
    static ɵfac: i0.ɵɵFactoryDeclaration<LinkTranspiler, [null, { optional: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<LinkTranspiler>;
}
/** Token that represents the closing tag of a link. */
export declare const LINK_END: {};
/**
 * Class used for modeling the link start tokens.
 */
export declare class LinkStart {
    /** Key of the parameter which stores the link model. */
    readonly parameterKey: string;
    /**
     * Creates a new `LinkStart` that references the specified parameter.
     */
    constructor(
    /** Key of the parameter which stores the link model. */
    parameterKey: string);
}
